
/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 */



#ifndef IMAGE_HPP
#define IMAGE_HPP

#include <color.hpp>

#include <vector>


namespace cpe
{
/** Storage class for an image
 *  Store image as a vector of color
 *
*/
class image
{
public:

    // ********************************************* //
    // ********************************************* //
    //  CONSTRUCTORS
    // ********************************************* //
    // ********************************************* //

    /** \brief empty constructor */
    image();
    /** \brief direct allocation of empty image (black image) */
    image(unsigned int Nx,unsigned int Ny);
    /** \brief direct set of image with the raw data
     *  \pre size of data must be Nx*Ny
     */
    image(unsigned int Nx,unsigned int Ny,const std::vector<color> data);

    // ********************************************* //
    // ********************************************* //
    //  Size of image
    // ********************************************* //
    // ********************************************* //

    /** \brief resize a new black image to the given size */
    void resize(unsigned int Nx,unsigned int Ny);

    /** \brief get width */
    unsigned int Nx() const;
    /** \brief get height */
    unsigned int Ny() const;


    // ********************************************* //
    // ********************************************* //
    //  Color access in picture
    // ********************************************* //
    // ********************************************* //

    /** \brief Fill the full image with the given color */
    void fill(const color& c);
    /** \brief Fill a square bloc of data with the given color
     *  \pre All indices spaned the bloc must belong to the image
     */
    void fill_bloc(unsigned int kx,unsigned int ky,unsigned int size_bloc,const color& c);

    /** \brief get the color of the given indices
     *  \pre (kx,ky) must be within the size of the image  */
    const color& get(const unsigned int kx,const unsigned ky) const;

    /** \brief set the color at the given indice
     *  \pre (kx,ky) must be within the size of the image  */
    void set(const unsigned int kx,const unsigned ky,const color& c);



    /** \brief get the color of the given indices
     *  \pre (kx,ky) must be within the size of the image  */
    const color& operator()(const unsigned int kx,const unsigned int ky) const;
    /** \brief get/set the color of the given indices
     *  \pre (kx,ky) must be within the size of the image  */
    color& operator()(const unsigned int kx,const unsigned int ky);


    // ********************************************* //
    // ********************************************* //
    //  Transfert of color from other image
    // ********************************************* //
    // ********************************************* //

    /** \brief copy the bloc of the source image to the destination image at the given indices
     *  \pre the indices must be within the size of the images */
    static void copy_bloc(unsigned int kx_source,unsigned int ky_source,const image& image_source,
                          unsigned int kx_destination,unsigned int ky_destination,image* image_destination,
                          unsigned int size_bloc);


    // ********************************************* //
    // ********************************************* //
    //  Raw access to the data
    // ********************************************* //
    // ********************************************* //

    /** \brief Directly set the raw data as a vector of concatenated (r,g,b) values
     *  \pre The size of the raw data must be 3*Nx*Ny
     */
    void set_raw_data(const std::vector<octet>& raw_data);
    /** \brief Export the data into a vector of concatenated rgba values */
    std::vector<octet> to_raw_data_rgba() const;
    /** \brief Export the data into a vector of concatenated rgb values */
    std::vector<octet> to_raw_data_rgb() const;




private:
    /** \brief internal storage of width */
    unsigned int size_Nx;
    /** \brief internal storage of height */
    unsigned int size_Ny;

    /** \brief internal storage of data
     *
     * Format: (r0,b0,g0,r1,b1,g1,...,rn,gn,bn) */
    std::vector<color> data;
};
}

#endif
